/****************************************************************
**
** Simulation for microphone beams
**
****************************************************************/

#include <qapplication.h>
#include <qpushbutton.h>
#include <qscrollbar.h>
#include <qpainter.h>
#include <qfont.h>
#include <stdio.h>
#include <qcolor.h>
#include <math.h>
#include "micpos.h"
#include "main.h"

#define DIST 5000

DrawArea::DrawArea(QCheckBox ** cb,  QWidget *parent, const char *name )
        : QWidget( parent, name )
{
    int i;
    d=0;
    a=0;
    checkbox=cb;
    for (i=0; i<MICNUM; i++) {
	connect(checkbox[i], SIGNAL(clicked()), this, SLOT(update()));
    }
    update();
}

void DrawArea::setDelay(int delay)
{
    d=float(delay)/100.0;
    update();
}

void DrawArea::setAngle(int angle)
{
    a=float(angle)/180*M_PI;
    update();
}

void DrawArea::resizeEvent( QResizeEvent *)
{
}

void DrawArea::paintEvent( QPaintEvent *)
{
    typedef struct {
      double x,y;
    } Tupel;
    int frequencies[10]={100, 200, 300, 500, 750, 1000, 1500, 2000, 3000, 5000};
    int i,k,freq,f;
    double length[MICNUM];
    int xpaint[180],ypaint[180];
    double amp, ampsin, ampcos;
    char s[10];
    Tupel soundpos;
    QPainter paint(this);
    QWMatrix matrix;
    matrix.translate(250,200);
    matrix.scale(1,1);
    paint.setWorldMatrix(matrix);
    QColor c;
    c.setRgb(0,0,0);
    paint.setPen(c);
    paint.setFont( QFont( "Times", 14, QFont::Bold ));
    for (i=0; i<MICNUM; i++) {
	paint.drawEllipse(int(micpos[i].x*0.2-5),int(micpos[i].y*0.2-5),10,10);
    }
    // for each of the above frequencies...
    for (freq=0; freq<11; freq++) {
	// put a virtual sound source in a distance of DIST mm and
	// turn it round the microphone array
	for (i=0; i<180; i++) {
	    soundpos.x=cos(float(i)/90.0*M_PI)*DIST;
	    soundpos.y=sin(float(i)/90.0*M_PI)*DIST;
	    // calculate the distance from the source to each microphone
	    if (freq==10) {
		amp=0;
		for (k=0; k<MICNUM; k++) {
		    length[k]=hypot(soundpos.x-micpos[k].x,soundpos.y-micpos[k].y);
		    // modify length with selected delay
		    length[k]+=d*(cos(a)*micpos[k].x+sin(a)*micpos[k].y);
		}
		for (f=300; f<3400; f+=100) {
		    ampsin=0; ampcos=0;
		    for (k=0; k<MICNUM; k++) {
			// calculate sine and cosine part of the incoming wave
			if ((*checkbox[k]).isChecked()) {
			ampsin-=sin(length[k]/SPEEDOFSOUND*2.0*M_PI*float(f))/31.0;
			ampcos-=cos(length[k]/SPEEDOFSOUND*2.0*M_PI*float(f))/31.0;
		      } else {
			ampsin+=sin(length[k]/SPEEDOFSOUND*2.0*M_PI*float(f))/31.0;
			ampcos+=cos(length[k]/SPEEDOFSOUND*2.0*M_PI*float(f))/31.0;
		      }
		    }
		    // add to overall amplitude
		    amp+=hypot(ampsin,ampcos);
		}
	    }
	    else {
		ampsin=0; ampcos=0;
		for (k=0; k<MICNUM; k++) {
		    length[k]=hypot(soundpos.x-micpos[k].x,soundpos.y-micpos[k].y);
		    // modify length with selected delay
		    length[k]+=d*(cos(a)*micpos[k].x+sin(a)*micpos[k].y);
		    // calculate sine and cosine part of the incoming wave
		    if ((*checkbox[k]).isChecked()) {
		      ampsin-=sin(length[k]/SPEEDOFSOUND*2.0*M_PI*frequencies[freq]);
		      ampcos-=cos(length[k]/SPEEDOFSOUND*2.0*M_PI*frequencies[freq]);
		    } else {
		      ampsin+=sin(length[k]/SPEEDOFSOUND*2.0*M_PI*frequencies[freq]);
		      ampcos+=cos(length[k]/SPEEDOFSOUND*2.0*M_PI*frequencies[freq]);
		      
		    }

		}
		amp=hypot(ampsin,ampcos);
	    }
	    xpaint[i]=int((200.0/(MICNUM*MICNUM))*cos(double(i)/90.0*M_PI)*amp*amp);
	    ypaint[i]=int((200.0/(MICNUM*MICNUM))*sin(double(i)/90.0*M_PI)*amp*amp);
	}
	// set rainbow-colors for the frequencies and black for the sum
	if (freq==10) {c.setRgb(0,0,0);} else {c.setHsv(36*freq,255,255);}
	paint.setPen(c);
	for (i=0; i<180; i++) {
	    paint.drawLine(xpaint[i],ypaint[i],xpaint[(i+1)%180],ypaint[(i+1)%180]);
	}
	// write the corresponding text
	if (freq==10) {sprintf(s,"300-3300 Hz");} 
	else {sprintf(s,"%4d Hz",frequencies[freq]);}
	paint.drawText(220,-200+30+freq*30,100,20,AlignRight,s);
    }
    // print delay
    sprintf(s,"%3.0f %%",d*100);
    c.setRgb(0,0,0);
    paint.setPen(c);
    paint.drawText(-240,185,60,15,AlignRight,s);
}


MainWidget::MainWidget( QWidget *parent, const char *name )
        : QWidget( parent, name )
{
    int i;
    char s[4];
    // set fixed size of window
    setFixedSize( 620, 410+20+16+20 );

    // place the checkboxes for the signal-inverting
    for (i=0; i<MICNUM; i++) {
	sprintf(s,"%c inv.",'A'+i);
	checkbox[i] = new QCheckBox(s,this,"Checkbox");
	checkbox[i]->setGeometry(500*i/MICNUM+100,5,500/MICNUM,20);
    }

    // generate a draw area
    drawarea = new DrawArea(checkbox, this, "Draw Area");
    drawarea->setGeometry(5,30,590,440);
    drawarea->setMaximumSize(600,410);
    QColor c;
    c.setRgb(220,220,220);
    drawarea->setBackgroundColor(c);

    // place the Quit-button
    quit = new QPushButton( "Quit", this, "quit" );
    quit->setGeometry( 5, 5, 50, 20 );
    quit->setFont( QFont( "Times", 14, QFont::Bold ) );
    connect( quit, SIGNAL(clicked()), qApp, SLOT(quit()) );


    // generate a horizontal scrollbar for the delay-time
    hBar = new QScrollBar( -200, 200,		       	// range
			   1, 10, 			// line/page steps
			   0, 				// inital value
			   QScrollBar::Horizontal, 	// orientation
                           this, "hscrollbar" );
    hBar->setGeometry(5,445,590,15);
    connect( hBar, SIGNAL(valueChanged(int)), drawarea , SLOT(setDelay(int)) );

    // generate a vertical scollbar for the angle
    vBar= new QScrollBar( -180, 180,		       	// range
			   1, 10, 			// line/page steps
			   0, 				// inital value
			   QScrollBar::Vertical, 	// orientation
                           this, "vscrollbar" );
    vBar->setGeometry(600,30,15,410);
    connect( vBar, SIGNAL(valueChanged(int)), drawarea , SLOT(setAngle(int)) );
}


void MainWidget::resizeEvent( QResizeEvent * )
{
}

int main( int argc, char **argv )
{
    QApplication a( argc, argv );
    MainWidget w;
    a.setMainWidget( &w );
    w.show();
    return a.exec();
}
